@@ -17,7 +17,7 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2005-2013 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005-2013, 2016, 2017 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2008 Kjartan Maraas <kmaraas@gnome.org>
 // Copyright (C) 2008 Boris Toloknov <tlknv@yandex.ru>
 // Copyright (C) 2008 Haruyuki Kawabe <Haruyuki.Kawabe@unisys.co.jp>
@@ -25,7 +25,7 @@
 // Copyright (C) 2009 Warren Toomey <wkt@tuhs.org>
 // Copyright (C) 2009, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009 Reece Dunn <msclrhd@gmail.com>
-// Copyright (C) 2010, 2012 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2010, 2012, 2013 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2010 OSSD CDAC Mumbai by Leena Chourey (leenac@cdacmumbai.in) and Onkar Potdar (onkar@cdacmumbai.in)
 // Copyright (C) 2011 Joshua Richardson <jric@chegg.com>
@@ -37,6 +37,8 @@
 // Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2013 Julien Nabet <serval2412@yahoo.fr>
 // Copyright (C) 2013 Johannes Brandstätter <jbrandstaetter@gmail.com>
+// Copyright (C) 2014 Fabio D'Urso <fabiodurso@hotmail.it>
+// Copyright (C) 2016 Vincent Le Garrec <legarrec.vincent@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -1327,7 +1329,7 @@ void HtmlOutputDev::drawJpegImage(GfxState *state, Stream *str)
   // open the image file
   GooString *fName=createImageFileName("jpg");
   if (!(f1 = fopen(fName->getCString(), "wb"))) {
-    error(errIO, -1, "Couldn't open image file '%s'", fName->getCString());
+    error(errIO, -1, "Couldn't open image file '{0:t}'", fName);
     delete fName;
     return;
   }
@@ -1361,7 +1363,7 @@ void HtmlOutputDev::drawPngImage(GfxState *state, Stream *str, int width, int he
   // open the image file
   GooString *fName=createImageFileName("png");
   if (!(f1 = fopen(fName->getCString(), "wb"))) {
-    error(errIO, -1, "Couldn't open image file '%s'", fName->getCString());
+    error(errIO, -1, "Couldn't open image file '{0:t}'", fName);
     delete fName;
     return;
   }
@@ -1369,7 +1371,7 @@ void HtmlOutputDev::drawPngImage(GfxState *state, Stream *str, int width, int he
   PNGWriter *writer = new PNGWriter( isMask ? PNGWriter::MONOCHROME : PNGWriter::RGB );
   // TODO can we calculate the resolution of the image?
   if (!writer->init(f1, width, height, 72, 72)) {
-    error(errInternal, -1, "Can't init PNG for image '%s'", fName->getCString());
+    error(errInternal, -1, "Can't init PNG for image '{0:t}'", fName);
     delete writer;
     fclose(f1);
     return;
@@ -1391,6 +1393,15 @@ void HtmlOutputDev::drawPngImage(GfxState *state, Stream *str, int width, int he
 
       // Convert into a PNG row
       p = imgStr->getLine();
+      if (!p) {
+        error(errIO, -1, "Failed to read PNG. '{0:t}' will be incorrect", fName);
+        delete fName;
+        gfree(row);
+        delete writer;
+        delete imgStr;
+        fclose(f1);
+        return;
+      }
       for (int x = 0; x < width; x++) {
         colorMap->getRGB(p, &rgb);
         // Write the RGB pixels into the row
@@ -1401,7 +1412,7 @@ void HtmlOutputDev::drawPngImage(GfxState *state, Stream *str, int width, int he
       }
 
       if (!writer->writeRow(row_pointer)) {
-        error(errIO, -1, "Failed to write into PNG '%s'", fName->getCString());
+        error(errIO, -1, "Failed to write into PNG '{0:t}'", fName);
         delete writer;
         delete imgStr;
         fclose(f1);
@@ -1413,32 +1424,39 @@ void HtmlOutputDev::drawPngImage(GfxState *state, Stream *str, int width, int he
     delete imgStr;
   }
   else { // isMask == true
-    ImageStream *imgStr = new ImageStream(str, width, 1, 1);
-    imgStr->reset();
+    int size = (width + 7)/8;
 
-    Guchar *png_row = (Guchar *)gmalloc( width );
+    // PDF masks use 0 = draw current color, 1 = leave unchanged.
+    // We invert this to provide the standard interpretation of alpha
+    // (0 = transparent, 1 = opaque). If the colorMap already inverts
+    // the mask we leave the data unchanged.
+    int invert_bits = 0xff;
+    if (colorMap) {
+      GfxGray gray;
+      Guchar zero = 0;
+      colorMap->getGray(&zero, &gray);
+      if (colToByte(gray) == 0)
+        invert_bits = 0x00;
+    }
+
+    str->reset();
+    Guchar *png_row = (Guchar *)gmalloc(size);
 
     for (int ri = 0; ri < height; ++ri)
     {
-      // read the row of the mask
-      Guchar *bit_row = imgStr->getLine();
-
-      // invert for PNG
-      for(int i = 0; i < width; i++)
-        png_row[i] = bit_row[i] ? 0xff : 0x00;
+      for(int i = 0; i < size; i++)
+        png_row[i] = str->getChar() ^ invert_bits;
 
       if (!writer->writeRow( &png_row ))
       {
-        error(errIO, -1, "Failed to write into PNG '%s'", fName->getCString());
+        error(errIO, -1, "Failed to write into PNG '{0:t}'", fName);
         delete writer;
         fclose(f1);
-        delete imgStr;
         gfree(png_row);
         return;
       }
     }
-    imgStr->close();
-    delete imgStr;
+    str->close();
     gfree(png_row);
   }
 
@@ -1620,7 +1638,9 @@ GooString* HtmlOutputDev::getLinkDest(AnnotLink *link){
 		      file->append(".html");
 		  }
 		  file->append('#');
-		  file->append(GooString::fromInt(page));
+		  GooString *pgNum = GooString::fromInt(page);
+		  file->append(pgNum);
+		  delete pgNum;
 	      }
 	  }
 	  if (printCommands && file) printf("filename %s\n",file->getCString());
